Index: channels/chan_sip.c
===================================================================
--- channels/chan_sip.c	(revision 244767)
+++ channels/chan_sip.c	(working copy)
@@ -16429,46 +16429,30 @@
 	.read = function_sipchaninfo_read,
 };
 
-static int read_to_parts(struct sip_pvt *p, struct sip_request *req, char **name, char **number)
+static int get_redirect_to_parts(const char *to_hdr, char **name, char **number)
 {
-
 	char to_header[256];
-	char *to_name = NULL;
+	char to_name[50] = { 0, };
 	char *to_number = NULL;
-	char *separator;
+	char *domain = NULL; /* don't care about domain, but need pointer for parse_uri to work correctly */
 
-	ast_copy_string(to_header, get_header(req, "To"), sizeof(to_header));
+	ast_copy_string(to_header, to_hdr, sizeof(to_header));
 
-	/* Let's get that number first! */
+	/* strip the display-name portion off the beginning of the TO header. */
+	get_calleridname(to_header, to_name, sizeof(to_name));
+
+	/* get uri within < > brackets */
 	to_number = get_in_brackets(to_header);
 
-	if (!strncasecmp(to_number, "sip:", 4)) {
-		to_number += 4;
-	} else if (!strncasecmp(to_number, "sips:", 5)) {
-		to_number += 5;
-	} else {
-		ast_log(LOG_WARNING, "Not a SIP URI? (%s)!\n", to_number);
+	/* parse out the number here */
+	if (parse_uri(to_number, "sip:,sips:", &to_number, NULL, &domain, NULL, NULL)) {
+		ast_log(LOG_ERROR, "to header can not be parsed \n");
 		return -1;
 	}
 
-	/* Remove the host and such since we just want the number */
-	if ((separator = strchr(to_number, '@'))) {
-		*separator = '\0';
-	}
-
-	/* We have the number. Let's get the name now. */
-
-	if (*to_header == '\"') {
-		to_name = to_header + 1;
-		if (!(separator = (char *)find_closing_quote(to_name, NULL))) {
-			ast_log(LOG_NOTICE, "No closing quote in name section of To: header (%s)\n", to_header);
-			return -1;
-		}
-		*separator = '\0';
-	}
-
 	if (number) {
 		*number = ast_strdup(to_number);
+		SIP_PEDANTIC_DECODE(*number);
 	}
 	if (name && !ast_strlen_zero(to_name)) {
 		*name = ast_strdup(to_name);
@@ -16493,7 +16477,7 @@
 	res = get_rdnis(p, req, &redirecting_from_name, &redirecting_from_number, &reason);
 	if (res == -1) {
 		if (is_response) {
-			read_to_parts(p, req, &redirecting_from_name, &redirecting_from_number);
+			get_redirect_to_parts(get_header(req, "TO"), &redirecting_from_name, &redirecting_from_number);
 		} else {
 			return;
 		}
@@ -16506,7 +16490,7 @@
 	if (is_response) {
 		parse_moved_contact(p, req, &redirecting_to_name, &redirecting_to_number, set_call_forward);
 	} else {
-		read_to_parts(p, req, &redirecting_to_name, &redirecting_to_number);
+		get_redirect_to_parts(get_header(req, "TO"), &redirecting_to_name, &redirecting_to_number);
 	}
 
 	if (!ast_strlen_zero(redirecting_from_number)) {
